﻿#  -*- coding: utf-8 -*-
# ln -s /usr/script/ovpn.sh /etc/rcS.d/ovpn.sh Autostart
#  OpenVPNManager
#  
#  https://board.dreamboxtools.de/
#  
#  http://board.dreambox-tools.info/member.php?24108-koepi
#
# This plugin is open source but it is NOT free software.
#
# This plugin may only be distributed to and executed on hardware which
# is licensed by Dream Multimedia GmbH.
# In other words:
# It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
# to hardware which is NOT licensed by Dream Multimedia GmbH.
# It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
# on hardware which is NOT licensed by Dream Multimedia GmbH.
#
# If you want to use or modify the code or parts of it,
# you have to keep MY license and inform me about the modifications by mail.
#
from enigma import eTimer, eConsoleAppContainer, ePicLoad
from Components.ActionMap import ActionMap
from Components.Pixmap import Pixmap
from Tools.LoadPixmap import LoadPixmap
from Components.Label import Label
from Components.ScrollLabel import ScrollLabel
from Components.Sources.StaticText import StaticText
from Components.Button import Button
from Components.ConfigList import ConfigListScreen
from Components.config import config, ConfigSubsection, getConfigListEntry, ConfigSet, ConfigFloat, ConfigInteger, ConfigIP, ConfigText, ConfigOnOff
from Plugins.Plugin import PluginDescriptor
from Screens.Console import Console
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Tools.Directories import fileExists, resolveFilename, SCOPE_SKIN
from os import system as sh
from datetime import datetime
from subprocess import Popen, PIPE, STDOUT
from os import system as os_system
from Screens.Standby import TryQuitMainloop
from ipinfo import get_ip_info
import ping
import time
import re
import os

config.plugins.OpenVpnManager = ConfigSubsection()
config.plugins.OpenVpnManager.ServerIP = ConfigIP(default=[10,8,0,1])
config.plugins.OpenVpnManager.delay = ConfigInteger(default = 5, limits = (1, 10))
config.plugins.OpenVpnManager.Timeout = ConfigText(default = "0.20")
config.plugins.OpenVpnManager.autostart = ConfigOnOff(default = False)

ip = "%d.%d.%d.%d" % tuple(config.plugins.OpenVpnManager.ServerIP.value)
delay = config.plugins.OpenVpnManager.delay.value
TimeoutSek = float(config.plugins.OpenVpnManager.Timeout.value)
autostart = config.plugins.OpenVpnManager.autostart.value
pathvpn = "/etc/openvpn/"
pathpic = "/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/icons/"


Version = "V 3.0"
cam = "openvpn"
def main(session, **kwargs):
	session.open(OpenVpnManager)
	
def Plugins(**kwargs):
	return [PluginDescriptor(
		name="OpenVpnManager",
		description=_("Starten / Stoppen / Überwachen"),
		where = [PluginDescriptor.WHERE_PLUGINMENU, PluginDescriptor.WHERE_EXTENSIONSMENU],
		icon="openvpn.png",
		fnc = main)]	

class OpenVpnManager(Screen):
	skin = """
		<screen backgroundColor="#ff141414" flags="wfNoBorder" name="OpenVpnManager" position="0,0" size="1920,1080" title="OpenVpnManager">
		<eLabel backgroundColor="#00011a42" cornerRadius="20" foregroundColor="#ff141414" position="340,65" size="1270,65" zPosition="-10" />
		<eLabel backgroundColor="#00011a42" cornerRadius="20" foregroundColor="#ff141414" position="340,135" size="1270,830" zPosition="-10" />
		<eLabel backgroundColor="#00011a42" cornerRadius="20" foregroundColor="#ff141414" position="340,970" size="1270,65" zPosition="-10" />
		<eLabel backgroundColor="#00141414" cornerRadius="20" foregroundColor="#ff141414" position="338,63" size="1274,69" zPosition="-15" />
		<eLabel backgroundColor="#00141414" cornerRadius="20" foregroundColor="#ff141414" position="338,133" size="1274,834" zPosition="-15" />
		<eLabel backgroundColor="#00141414" cornerRadius="20" foregroundColor="#ff141414" position="338,968" size="1274,69" zPosition="-15" />
		<widget backgroundColor="#00011a42" font="Regular; 30" foregroundColor="#00cf8106" halign="left" position="420,77" render="Label" size="700,40" source="Title" transparent="1" valign="center" zPosition="2" />
		<widget backgroundColor="#00011a42" font="Thin; 28" foregroundColor="#00888888" halign="right" position="1140,77" render="Label" size="450,40" source="global.CurrentTime" transparent="2" valign="center">
			<convert type="ClockToText">Format: %A, %d.%m.%Y  %H:%M:%S</convert>
		</widget>
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/red.png"      position= "375,990" size="10,25" alphatest="on" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/green.png"    position= "675,990" size="10,25" alphatest="on" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/yellow.png"   position= "975,990" size="10,25" alphatest="on" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/blue.png"     position="1275,990" size="10,25" alphatest="on" />	
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/key_menu.png" position="1480,989" size="40,30" alphatest="on" transparent="1" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/key_ok.png"	 position="1550,984" size="40,40" alphatest="on" transparent="1" />
		<widget backgroundColor="#00011a42" font="Thin; 28" foregroundColor="#00eeeeee" halign="left" position="395,983" render="Label" size="260,40" source="key_red" transparent="1" valign="center" zPosition="2" />
		<widget backgroundColor="#00011a42" font="Thin; 28" foregroundColor="#00eeeeee" halign="left" position="695,983" render="Label" size="260,40" source="key_green" transparent="1" valign="center" zPosition="2" />
		<widget name="OpenVpnPng" position="490,200" size="160,160" alphatest="on" transparent="1" zPosition="2"/>
		<widget name="warte"      position="370,400" size="400, 40" font="Regular;32" halign="center" zPosition="2" foregroundColor="#00389416" backgroundColor="#00011a42"/>
		<widget name="ownIP"      position="370,450" size="400, 40" font="Regular;32" halign="center" zPosition="2" foregroundColor="#000064c7" backgroundColor="#00011a42"/>
		<widget name="stat"       position="370,500" size="400, 40" font="Regular;32" halign="center" zPosition="2" foregroundColor="#000064c7" backgroundColor="#00011a42"/>
		<widget name="autostart"  position="450,550" size="180, 40" font="Regular;32" halign="center" zPosition="3" foregroundColor="#000064c7" backgroundColor="#00011a42"/>
		<widget name="statusPic"  position="630,553" size= "26, 26" alphatest="on" transparent="3" zPosition="2"/>
		<widget name="provider"   position="920,140" size="670,340" foregroundColor="#00ffffff" font="Regular; 28" valign="bottom" halign="left" zPosition="2" backgroundColor="#00011a42"/>
		<widget name="Vpn"        position="920,530" size="430,400" font="Regular;28" zPosition="2" noWrap="1" backgroundColor="#00011a42"/>
		<widget name="key_red"    position=  "0,460" size="140, 40" zPosition="3" font="Regular;20" halign="center" valign="center" foregroundColor="#00ffffff" transparent="1" shadowColor="black" shadowOffset="-2,-2"/>
		<widget name="key_green"  position="155,460" size="140, 40" zPosition="3" font="Regular;20" halign="center" valign="center" foregroundColor="#00ffffff" transparent="1" shadowColor="black" shadowOffset="-2,-2"/>
		</screen>"""
	
	def __init__(self, session, args = 0):
		Screen.__init__(self, session)
		self.session = session
		
		self.onShown.append(self.setStatePixmap)
		self.loop = eTimer()
		self["OpenVpnPng"] = Pixmap()
		self["warte"] = Label("")
		self["ownIP"]  = Label("")
		self["stat"]  = Label("")
		self["Vpn"]   = Label("")
		self["provider"] = Label("")
		self["autostart"] = Label("")
		self["statusPic"] = Pixmap()
		
		self["key_red"] = StaticText(_("Stop"))
		self["key_green"] = StaticText(_("Start"))
		self["key_blue"] = StaticText(_(" "))
		picload = ePicLoad()
		
		self["OpenVPNManagerActionMap"] = ActionMap(["OkCancelActions","ColorActions","DirectionActions","EPGSelectActions","MenuActions"],
		{
			"menu":	  self.openConfig,
			"red": 	  self.stopOpenVPN,
			"green":  self.startOpenVPN,
			"ok":	  self.weiter,
			"cancel": self.cancel
		}, -1)

		self.icon_start = LoadPixmap(cached=True, path=resolveFilename\
		(SCOPE_SKIN, "Merlin/green.png"))
		self.icon_stop  = LoadPixmap(cached=True, path=resolveFilename\
		(SCOPE_SKIN, "Merlin/red.png"))

		self.setTitle(_("OpenVpnManager  ") + Version)

		self.onLayoutFinish.append(self.provider)
		
	def setStatePixmap(self):
		self["autostart"].setText("Autostart")
		self["statusPic"].instance.setPixmapFromFile(pathpic + "/%s.png" %("green" if config.plugins.OpenVpnManager.autostart.value else "red"))
		cmd = None
		if config.plugins.OpenVpnManager.autostart.value:
			cmd = "systemctl enable %s" % (cam)
			print '______________', cmd
			print '[openvpn] handleCam returns:', os_system(cmd)
		else:
			cmd = "systemctl stop %s && systemctl disable %s" % (cam, cam)
			print '______________', cmd
			print '[openvpn] handleCam returns:', os_system(cmd)
#-----------------------------------------------------------------------------------

	def provider(self):
		info = "dummy"
		info = get_ip_info()
		self["provider"].setText(info)
		self.statik()
		
#-----------------------------------------------------------------------------------

	def startOpenVPN(self):
		self.handleCam(cam, 'start')
		with open(pathvpn+"startstop.list", "wb") as ovpn:
			ovpn.write("start")
		aktuell = datetime.strftime(datetime.now(), '%a %d %b %Y %H:%M'+ " Uhr")
		with open(pathvpn+"startzeit.uhr", "wb") as ovpn:
			ovpn.write(aktuell)
		self.timer()
		self.statik()
	
	def stopOpenVPN(self):
		with open(pathvpn+"startstop.list", "wb") as ovpn:
			ovpn.write("stop")
		self.handleCam(cam, 'stop')
		self.timer()
		self.statik()
		
	def cancel(self):
		self.close(None)

	def handleCam(self, cam, state):
		cmd = None
		autostart = config.plugins.OpenVpnManager.autostart.value
		print "=====> autostart 187", autostart
		if state == 'start':
			sh("/etc/init.d/%s start" % (cam))
			if	fileExists(pathvpn+"openvpn.status"):
				print "alles gut"
			else:
				print "===============> Schreib dein File"
				os.system("touch /etc/openvpn/openvpn.status")
		else:
			sh("/etc/init.d/%s stop" % (cam))
		self.timer()
		self.statik()

#--------------------- is OpenVpn online ------------------------------------------
		
	def	online(self,cam, state):
		r = ping.doOne(ip,TimeoutSek)
		if r == None or r > TimeoutSek:								#	Timeout
			self.handelCam(cam, stat)
		else:														#	Ping erfolgreich
			self["stat"].setText("Ping erfolgreich")
			self.statik()

#--------------------- Status der Verbindung ------------------------------------------			

	def statik(self):
		self["OpenVpnPng"].instance.setPixmapFromFile(pathpic+"vpn_off.png")
		vpnip = Popen("ifconfig tun0", shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True).stdout.read()
		x = vpnip.find("addr:")
		y = vpnip.find("P-t-P")
		if x >0:
			ownIP = vpnip[x+5:y]
			self["ownIP"].setText(''.join("Zugewiesene IP  "+ ownIP))
		else:
			ownIP = "no IP"
			self["ownIP"].setText(''.join("Zugewiesene IP  "+ ownIP))
		if	fileExists(pathvpn+"openvpn.status"):
			ovpn = file(pathvpn+"openvpn.status")
			r = ping.doOne(ip,TimeoutSek)
			if	r == None or r > TimeoutSek:
				self["warte"].setText("OpenVpn ist nicht gestartet")	# keine Verbindung
				txt = "Ping {}".format(ip)
				ovpn = file("/etc/openvpn/openvpn.status")
				self["stat"].setText(''.join(txt+" nicht erfolgreich"))
				self["Vpn"].setText(''.join(ovpn))
				self["OpenVpnPng"].instance.setPixmapFromFile(pathpic+"vpn_off.png")
			else:
				self["warte"].setText("OpenVpn ist gestartet")			# Verbindung
				txt = "Ping nach {}".format(ip)
				self["stat"].setText(''.join(txt+" erfolgreich"))
				self["OpenVpnPng"].instance.setPixmapFromFile(pathpic+"vpn_on.png")
				ovpn = file("/etc/openvpn/openvpn.status")
				self["Vpn"].setText(''.join(ovpn))
				self.wiederholer()
		else:
			self["Vpn"].setText("K e i n   S t a t u s - F i l e   a n g e l e g t !")

	def vpnip(self):
		vpn = Popen("ifconfig tun0", shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True).stdout.read()
		x = vpn.find("addr:")
		y = vpn.find("P-t-P")
		if x >0:
			ip = vpn[x+5:y]
			self["ip"].setText(''.join("Zugewiesene IP  "+ ip))
			self["OpenVpnPng"].instance.setPixmapFromFile(pathpic+"vpn_on.png")
		else:
			ip = "no IP"
			self["ip"].setText(''.join("Zugewiesene IP  "+ ip))
			self["OpenVpnPng"].instance.setPixmapFromFile(pathpic+"vpn_off.png")
	
#--------------------- Timer ------------------------------------------
	def	wiederholer(self):									# eTimer start Status
		self.loop.startLongTimer(delay)
		self.loop_conn = self.loop.timeout.connect(self.statik)
			
	def timer(self):										# Timer wartet bis  Ovpn fertig ist
		time.sleep(2)

	def weiter(self):
		r = ping.doOne(ip,TimeoutSek)
		if	r == None or r > TimeoutSek or fileExists("/etc/openvpn/startzeit.uhr") is False:
			self.session.open(MessageBox,_("OpenVpn ist nicht gestartet \n oder \n'/etc/openvpn/startzeit.uhr'\n ist nicht geschrieben worden"), MessageBox.TYPE_INFO)
			return
		else:
			self.session.open(Ausgabe)
	def openConfig(self):
		self.session.open(SettingsOpenVpnManager)

class Ausgabe(Screen):
	skin = """
		<screen backgroundColor="#ff141414" flags="wfNoBorder" name="OpenVpnManager" position="0,0" size="1920,1080" title="OpenVpnManager">
		<eLabel backgroundColor="#00011a42" cornerRadius="20" foregroundColor="#ff141414" position="625, 65" size="670, 65" zPosition="-10" />
		<eLabel backgroundColor="#00011a42" cornerRadius="20" foregroundColor="#ff141414" position="625,135" size="670,230" zPosition="-10" />
		<eLabel backgroundColor="#00011a42" cornerRadius="20" foregroundColor="#ff141414" position="625,368" size="670, 65" zPosition="-10" />
		<eLabel backgroundColor="#00141414" cornerRadius="20" foregroundColor="#ff141414" position="625, 63" size="670, 69" zPosition="-15" />
		<widget name="datum"    position="725,80" size="460, 40" halign="center" font="Regular;28" backgroundColor="#00011a42"/>
		<widget name="read"     position="725,140" size="300,40" font="Regular;32" backgroundColor="#00011a42"/>
		<widget name="receive"  position="645,220" size="300,40" font="Regular;32" halign="left"  foregroundColor="#00389416" backgroundColor="#00011a42"/>
		<widget name="transver" position="645,280" size="300,40" font="Regular;32" halign="left"  foregroundColor="#000064c7" backgroundColor="#00011a42"/>
		<widget name="sum_rx"   position="920,220" size="360,40" font="Regular;32" halign="right" foregroundColor="#00389416" backgroundColor="#00011a42"/>
		<widget name="sum_tx"   position="920,280" size="360,40" font="Regular;32" halign="right" foregroundColor="#000064c7" backgroundColor="#00011a42"/>
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/key_exit.png" position="1220,382" size="40,40" alphatest="on" transparent="1" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/key_ok.png"	  position="1150,382" size="40,40" alphatest="on" transparent="1" />
		</screen>"""

	def __init__(self, session):
		Screen.__init__(self, session)
		self.loop = eTimer()
		Screen.__init__(self, session, list)
		self["error"]   = ScrollLabel("")
		self["read"]   = ScrollLabel("")
		self["datum"]   = ScrollLabel("")
		self["receive"]  = ScrollLabel("")
		self["transver"] = ScrollLabel("")
		self["sum_rx"] = ScrollLabel("")
		self["sum_tx"] = ScrollLabel("")
		self.onLayoutFinish.append(self.befor)		
		self["OpenVPNManagerActionMap"] = ActionMap(["OkCancelActions"],
		{
			"ok":	  self.cancel,
			"cancel": self.cancel
		}, -1)
		
		with open("/etc/openvpn/startzeit.uhr","r") as f:
			dat = f.read()
		self["datum"].setText(''.join("Am "+ dat + " gestartet"))
	
	def befor(self):
		global sek
		global rx_byte_befor
		global tx_byte_befor
		with open("/proc/net/dev", 'r') as file:
			info = file.readlines()
			for line in info:
				if line.startswith("  tun0"):
					data = line.split()
					if len(data)>0:
						rx_byte_befor = float(data[1])
						tx_byte_befor = float(data[9])

		self.wait = 5
		self.repater()

	def after(self):
		global rx_bit
		global tx_bit
		with open("/proc/net/dev", 'r') as file:
			info = file.readlines()
			for line in info:
				if line.startswith("  tun0"):
					data = line.split()
					if len(data)>0:
						rx_byte_after = float(data[1])
						tx_byte_after = float(data[9])
		rx_byte = (rx_byte_after - rx_byte_befor) # x 2 = 1Sek
		tx_byte = (tx_byte_after - tx_byte_befor)
		
		rx_byte = round(rx_byte, 0)
		tx_byte = round(tx_byte, 0)
		self["receive"].setText("Receive %0.0f Byte" % round(rx_byte, 0))
		self["transver"].setText("Transver %0.0f Byte" % round(tx_byte, 0))

		with open("/sys/devices/virtual/net/tun0/statistics/rx_bytes","r") as f:
			rx = float(f.read())
		if rx > 1048576:
			sum_rx = rx
			self["sum_rx"].setText("Total %0.3f MByte" % round((sum_rx/1048576),3))
		elif rx > 1024:
			sum_rx = rx
			self["sum_rx"].setText("Total %0.3f KByte" % round((sum_rx/1024),3))
		else:		
			sum_rx = rx
			self["sum_rx"].setText("Total %0.0f Byte" % round(sum_rx, 0))

		with open("/sys/devices/virtual/net/tun0/statistics/tx_bytes","r") as f:
			tx = float(f.read())
		if tx > 1048576:
			sum_tx = tx
			self["sum_tx"].setText("Total %0.3f MByte" % round((sum_tx/1048576),3))
		elif tx > 1024:
			sum_tx = tx
			self["sum_tx"].setText("Total %0.3f KByte" % round((sum_tx/1024),3))
		else:		
			sum_tx = tx
			self["sum_tx"].setText("Total %0.0f Byte" % round(sum_tx, 0))

		self.befor()
#--------------------- verschiedene Timer ------------------------------------------

	def repater(self): # zählt von 10 -> 0
		self["read"].setText("Read in %0.0f Sek" % (self.wait))
		self.wait -= 1
		if self.wait < 0:
			self.after()
		else:
			self.repater1()

	def repater1(self): # zählt von 10 -> 0
		self.loop.startLongTimer(1)
		self.loop_conn = self.loop.timeout.connect(self.repater)
	
	def cancel(self):
		self.loop.stop()
		self.close(None)			

class SettingsOpenVpnManager(Screen, ConfigListScreen):
	skin = """
		<screen backgroundColor="#ff141414" flags="wfNoBorder" name="OpenVpnManager Settings" position="0,0" size="1920,1080" title="OpenVpnManager Settings">
		<eLabel backgroundColor="#00011a42" cornerRadius="20" foregroundColor="#ff141414" position="340,65" size="1270,65" zPosition="-10" />
		<eLabel backgroundColor="#00011a42" cornerRadius="20" foregroundColor="#ff141414" position="340,135" size="1270,830" zPosition="-10" />
		<eLabel backgroundColor="#00011a42" cornerRadius="20" foregroundColor="#ff141414" position="340,970" size="1270,65" zPosition="-10" />
		<eLabel backgroundColor="#00141414" cornerRadius="20" foregroundColor="#ff141414" position="338,63" size="1274,69" zPosition="-15" />
		<eLabel backgroundColor="#00141414" cornerRadius="20" foregroundColor="#ff141414" position="338,133" size="1274,834" zPosition="-15" />
		<eLabel backgroundColor="#00141414" cornerRadius="20" foregroundColor="#ff141414" position="338,968" size="1274,69" zPosition="-15" />
		<widget backgroundColor="#00011a42" font="Regular; 30" foregroundColor="#00cf8106" halign="left" position="420,77" render="Label" size="700,40" source="Title" transparent="1" valign="center" zPosition="1" />
		<widget backgroundColor="#00011a42" font="Thin; 28" foregroundColor="#00888888" halign="right" position="1140,77" render="Label" size="450,40" source="global.CurrentTime" transparent="1" valign="center">
			<convert type="ClockToText">Format: %A, %d.%m.%Y  %H:%M:%S</convert>
		</widget>
		<widget name="config" position= "400,140" size="1150, 400" zPosition="2" backgroundColor="#00011a42"/>
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/red.png"    position= "375,990" size="10,25" alphatest="on" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/green.png"  position= "675,990" size="10,25" alphatest="on" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/yellow.png" position= "975,990" size="10,25" alphatest="on" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/OpenVpnManager/buttons/blue.png"   position="1275,990" size="10,25" alphatest="on" />	
		<widget backgroundColor="#00011a42" font="Thin; 28" foregroundColor="#00eeeeee" halign="left"  position= "395,983" size="260,40" render="Label" source="key_red" transparent="1" valign="center" zPosition="5" />
		<widget backgroundColor="#00011a42" font="Thin; 28" foregroundColor="#00eeeeee" halign="left"  position= "695,983" size="260,40" render="Label" source="key_green" transparent="1" valign="center" zPosition="5" />
		</screen>"""

	def __init__(self, session):
		Screen.__init__(self, session)
		self.setTitle(_("OpenVpnManager Settings"))


		self["actions"] = ActionMap(["SetupActions"],
			{
				"cancel": self.keyCancel,
				"save": self.saveEntry,
				"ok": self.saveEntry,
			}, -2)
		self.list = []
		self["key_red"] = StaticText(_("Cancel"))
		self["key_green"] = StaticText(_("OK"))

		self.list = []
		ConfigListScreen.__init__(self, self.list, session = session)
		self.createConfig()

	def createConfig(self,configElement=""):
		self.list = []
		self.list.append(getConfigListEntry(_("ServerIP"), config.plugins.OpenVpnManager.ServerIP))
		self.list.append(getConfigListEntry(_("Repeater"), config.plugins.OpenVpnManager.delay))
		self.list.append(getConfigListEntry(_("Ping in Sek"), config.plugins.OpenVpnManager.Timeout))
		self.list.append(getConfigListEntry(_("Autostart"), config.plugins.OpenVpnManager.autostart))
		self["config"].list = self.list

	def saveEntry(self):
		for x in self["config"].list:
			if len(x)>1:
				x[1].save()
		config.plugins.OpenVpnManager.autostart.save()
		self.close()

	def keyCancel(self):
		self.close()